/********************************************************************************************
*  Simulation to evaluate the relative importance of random variables using Akaike weights, *
*  standardized regression coefficients, partial and semi-partial correlation coefficients, *
*  and hierarchical partitioning.                                                           *
*																							*
*  REMEMBER to change subdirectory where hierpart.sas is called from include statement. 	*
*  kmb & mmc                                                                       09/24/08 *
********************************************************************************************/

%macro aicwts;
   filename junk dummy;
   proc printto log=junk;
   run;

* This section creates the data set;
   %do j=1 %to &files;
      data random1;
        ODS select none;
        file=&j;
        do case = 1 to 20000;
            y = rannor(0)+500;
           x1 = rannor(0);
           x2 = rannor(0);
           x3 = rannor(0);
           x4 = rannor(0);
           output;
         end;

         * Set the pattern of correlations among variables;
         proc iml;
           use random1;
           read all var {y x1 x2 x3 x4 file} into vars;
           varnames= {y x1 x2 x3 x4 file};
           R={1 0.265 0.374 0.495 0.495 0, 0.265 1 0 0 0 0, 0.374 0 1 0 0 0, 0.495 0 0 1 0 0, 0.495 0 0 0 1 0, 0 0 0 0 0 1};
           corvars=vars*root(R);
           create cordata from corvars [colname=varnames];
           append from corvars;
         run;
         quit;

         * Run all possible models;
         ODS output ParameterEstimates=parms; * output the various coefficients;
         proc reg data=cordata outest=models outstb;
	       model y = x1 x2 x3 x4 /stb aic selection=rsquare best=1000 pcorr2 scorr2;
           by file;
		   title1'Regression Analysis';
         run;
         quit;
         *ODS select none;

         * Append the coefficients to the parms file;
         %if &j=1 %then %do;
           data parms1;
              set parms;
              keep file variable standardizedest SemiCorrTypeII SqPartCorrTypeII;
         %end;
         %else %do;
            data parms2;
              set parms;
              keep file variable standardizedest SemiCorrTypeII SqPartCorrTypeII;
         %end;
         run;
         %if &j>1 %then %do;
           proc append base=parms1 data=parms2;
         %end;
         run;
         * Generate AICc values;
         data models;
           set models;
           aicc= _AIC_ +(2*(_P_+1)*(_P_+2))/(_EDF_-2);
         run;
         * Choose minimum AICc values for each set of models;
         proc means data=models min;
           var aicc;
           by file;
           output out=minaic min=minaic;
         run;
         * Drop the FREQ and TYPE columns from the minaic table;
         data minaic;
           set minaic;
           drop _freq_ _type_;
         run;
         * Add the minimum AICc values to the models table;
         data models;
           merge models minaic;
           by file;
         run;
         * Calculate delta AICc & Akaike weights values;
         data models;
           set models;
           delta = aicc - minaic;
           aicwt = exp(-.5*delta);
         run;
         * Sum the initial weights;
         proc means data=models sum;
           var aicwt;
           by file;
           output out=aicwt sum=aictot;
         run;
         * Drop the FREQ and TYPE columns from the aicwt table;
         data aicwt;
           set aicwt;
           drop _freq_ _type_;
         run;
         * Add the aicwt AICc values to the models table;
         data models;
           merge models aicwt;
           by file;
         run;
         * Calculate the final weights and clean up the models table;
         %if &j=1 %then %do;
           data models1;
             set models;
             weight = aicwt/aictot;
             drop _model_ _type_ _depvar_;
         %end;
         %else %do;
           data models2;
             set models;
             weight = aicwt/aictot;
             drop _model_ _type_ _depvar_;
         %end;
         run;
         * Append results to the models1 table;
         %if &j>1 %then %do;
           proc append base=models1 data=models2;
         %end;
         run;

	     %include 'C:\temp\hierpart.sas';  *rem to change subdir;
         %hierpart;
   %end; *j loop;

%mend aicwts;

%let files=5;  *rem to change to 2500 when done testing;
%aicwts;

proc printto;
ods select all;

proc sort data=effects1;
  by var;
run;
proc means data=effects1 noprint;
  var Independent size0;
  by var;
  output out=meaneffect;
run;
data meaneffect;
	set meaneffect;
    drop _freq_ _type_;
run;
proc print data=meaneffect;
  title1 'Simulation Results: Independent Effects and Zero-Order Correlations'; 
run;

proc sort data=parms1;
  by variable;
run;
proc means data=parms1 noprint;
  var StandardizedEst SemiCorrTypeII SqPartCorrTypeII;
  by variable;
  output out=meanbeta;
data meanbeta;
	set meanbeta;
	drop _freq_ _type_;
run;
proc print data=meanbeta;
  title1 'Simulation Results: Standardized Coefficients, Semi-Partial, and Partial Correlations'; 
run;
