/******************************************************************************
                    HIERARCHICAL PARTITIONING MACRO
                    
This macro executes hierarchical partitioning analysis as described by Chevan & Sutherland in 
American Statistician, 1991, Vol. 45, No. 2, pp. 90-96. This macro was written by Kim Murray Berger
and Mary M. Conner based on a Dominance Analysis macro written by Razia Azen & Robert Ceurvorst.  
Hierarchical Partitioning analysis quantifies the importance of each predictor as its average 
contribution to the model r-square, across all possible models.

NOTE:  This program is limited to at most 10 predictors!

------
STEPS:
------
1. In this macro, change the %hierpart line (which appears just below these 
instructions) according to the following directions:
%hierpart (data=_last_, dep=Y, indep='list of predictors', p=n_of_predictors);
      
Either p= OR indep= is required -- not both.  If both are specified, p is 
determined by counting the variables in the indep= list.

data=      SAS dataset to be used.  Default is last dataset created.

dep=       Name of dependent variable.  Default is Y.

indep=     List of predictors in quotes.
 OR
p=         No. of predictors IF and ONLY IF they are named X1-Xp,
           in which case indep= is not required.

2. Save this macro, and add the following two lines (below) to the
SAS program in which you've read the data set to be analyzed:
%include 'k:\sasmacro\hierpart.sas';  *** CHANGE TO PATH WHERE MACRO IS SAVED ***;
%hierpart;
******************************************************************************/
run; option nosource;
%macro hierpart (data=cordata, dep=y, p=4, indep='x1 x2 x3 x4');

%if &indep ne %then %do;
  %if %index(&indep,%str(%'))>0 or %index(&indep,%str(%"))>0 
    %then %let indep = %substr(&indep,2,%length(&indep)-2);
    %let p=1; 
  %do %while(%length(%scan(&indep,&p))>0);
    %let x&p = %scan(&indep,&p);
    %let p = %eval(&p+1);
  %end;
  %let p = %eval(&p-1);
%end;

%else %if &p > 0 %then %do; 
  %let indep=;
  %do i=1 %to &p; 
    %let indep=&indep x&i;
    %let x&i=x&i;
  %end;  
%end;

%LET DATANAME=&SYSLAST;
option nonotes;
DATA _NULL_;  FILE PRINT LINESLEFT=WITH;  CALL SYMPUT('WITH',WITH);
DATA _NULL_;
  FILE PRINT NOTITLES LINESLEFT=WITHOUT;
  CALL SYMPUT ('MTITLE',TRIM(LEFT(WITHOUT-&WITH+2)));
  CALL SYMPUT('NMODELS', 2**&p -1);
RUN;

data original; set &data;

data labels; set;  
  keep &indep; 
  length dvlabel $40;
  call label(&dep,dvlabel);
  call symput('dvl',trim(dvlabel));
  if _n_=1 then stop;  
run;

/***********************************************************************
     PART 1:  regression/hierarchical partitioning of the original data 
***********************************************************************/

option notes;
proc reg corr data=original outest=onereg (keep=_in_ _rsq_ &indep);
  model &dep=&indep / stb pcorr2 scorr2;
  model &dep=&indep / selection=adjrsq best=&nmodels;
 run; quit; option nonotes;

* order all subset models in lexicographical order;
data modelmat;
  set onereg; 
  %if %substr(&sysver,1,1)=8 %then if _n_>1;;
  %do i=1 %to &p;
    &&x&i=(&&x&i>.); 
  %end;
  if _IN_=. then delete;
  keep &indep _IN_ _RSQ_;

proc sort; by _IN_ %do i=1 %to &p; descending &&x&i %end;; run;

 %end;

/**********************************************
          DOMINANCE ANALYSIS TABLE  
**********************************************/
option notes;
proc IML;
  start;
  * read the subset models matrix into DOM;
  use modelmat;
  read all into DOM;
  close modelmat;

* dom contains, for each subset model, 1/0 values for the predictors,
  the number of predictors in (_IN_), and the model r2 (_RSQ_); 
  p=ncol(dom)-2;
  ncol=ncol(dom);
   
* dom is rearranged to contain, for each subset model, the number of predictors
  in and the r2 of the model, followed by 1/0 values for predictors in/out;
  dom1=dom[,1:p];  dom2=dom[,(p+1):ncol];  dom=dom2||dom1;  free dom1 dom2; 

* generate table of additional contributions; 
  null=J(1,ncol,0);
  dom=null//dom;
  nrow=nrow(dom);

* dom now contains a top row of zeros for the null model;
  full=J(1,p,99);  fullrsq=J(1,1,99);
  reduced=J(1,p,99);  redrsq=J(1,1,99);
  contrib=J(nrow,p,0);  * additional contributions matrix;

  do i=1 to nrow;	* for each model;
    do j=1 to p;    * for each predictor;

      if dom(|i,j+2|)=0 then do;  * if predictor is not in subset model;
        reduced=dom[i,3:ncol];    * the 1/0 row represents the reduced model;
        full=reduced;             * the full model is same as reduced model;

        do k=1 to p;          
          if k=j then full(|1,k|)=1;   * add the jth predictor to the full model;
        end;    

        do r=1 to nrow;                			* for each model;
          comp=dom[r,3:ncol];          			* comp is the 1/0 row of dom;
          if comp=full then fullrsq=dom[r,2];   * r2 of row is fullrsq, or; 
          if comp=reduced then redrsq=dom[r,2]; * r2 of row is redrsq;
        end;

        contrib(|i,j|)=fullrsq-redrsq;          * contrib is r2 difference;
	  end;	* if predictor not in subset model;

      else do;
        contrib(|i,j|)=.;         * if predictor is in model, contib is .;
      end;

    end;	* for each model;
  end;  	* for each predictor;

  contrib=dom||contrib;
  *contrib=contrib[1:nrow-1,];

  cols = {IN RSQ %do i=1 %to &p; &&X&i %end; %do i=1 %to &p; CP&i %end;};

  create rsqtable from contrib[colname=cols];
  append from contrib;
  close rsqtable;

  finish;
run; quit;

data rsqtable;
  set rsqtable;
  if IN ne &p;
run;

proc summary nway; var CP1--CP&p; class in;
  output out=avgcont (drop=_type_) mean=&indep;

proc means noprint;
  var &indep; 
  output out=meanc mean=;

data avgcont; set labels meanc (in=y) avgcont;
  if y then in=999;

proc transpose prefix=size out=meanc (rename=(SIZE999=Independent _NAME_=VAR)); id IN; var &indep; run;
proc sort; by descending Independent;

data effects;
  set meanc;
  Joint = Independent-size0;
  file = &j;
  run;

proc means data=effects noprint sum;
  var independent;
  by file;
  output out=total sum=Total;
run;

data effects;
  merge effects total;
  by file;
  drop _type_ _freq_;
  Indpct = (Independent/Total);

%if &j=1 %then %do;
  data effects1;
  set effects;
  keep file Var Independent Joint Indpct size0;
%end;
%else %do;
  data effects2;
  set effects;
  keep file Var Independent Joint Indpct size0;
%end;
run;

%if &j>1 %then %do;
  proc append base=effects1 data=effects2;
%end;
run;

%DONE: option notes _last_=&syslast;
%mend hierpart;

